//
// console.cpp - implementation of a simple console library
//   that emulates the Turbo C++ console functions.
//
#include <iostream>
#include <windows.h>
#include "console.h"

using namespace std;

// Set the text color, see constants in console.h
void textcolor(int color)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_SCREEN_BUFFER_INFO coninfo;

	// Get current color
	GetConsoleScreenBufferInfo(hConsole, &coninfo);

	// Turn off the foreground bits
	coninfo.wAttributes = coninfo.wAttributes & 0x00F0;

	color += coninfo.wAttributes;

	SetConsoleTextAttribute(hConsole, color);
}

// Set the background color
void textbackground(int color)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_SCREEN_BUFFER_INFO coninfo;

	// Get current color
	GetConsoleScreenBufferInfo(hConsole, &coninfo);

	// Turn off the background bits
	coninfo.wAttributes = coninfo.wAttributes & 0x000F;

	// Make this a background color
	color = color << 4;
	color += coninfo.wAttributes;

	SetConsoleTextAttribute(hConsole, color);
}

// Position the cursor
void gotoxy(char x, char y)
{
	COORD pos = { x, y };
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	SetConsoleCursorPosition(hConsole, pos);
}

// Unhide the cursor
void showcursor()
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_CURSOR_INFO cursorinfo;

	GetConsoleCursorInfo(hConsole, &cursorinfo);
	cursorinfo.bVisible = true;
	SetConsoleCursorInfo(hConsole, &cursorinfo);
}

// Hide the cursor
void hidecursor()
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_CURSOR_INFO cursorinfo;

	GetConsoleCursorInfo(hConsole, &cursorinfo);
	cursorinfo.bVisible = false;
	SetConsoleCursorInfo(hConsole, &cursorinfo);
}

// Clear the screen, clears text and colors
void clrscr(int bgcolor, int fgcolor, char fill)
{
	COORD pos = { 0, 0 };
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	DWORD num;
	int color = (bgcolor << 4) + fgcolor;

	FillConsoleOutputCharacter(hConsole,  fill, 80*25, pos, &num);
	FillConsoleOutputAttribute(hConsole, color, 80*25, pos, &num);
	gotoxy(0,0);
}

// A solid color box
void solidbox(char x, char y, char w, char h)
{
	// check parameters
	if ((x <  0) || (y <  0))
		return;
	if ((w < 1) || (h < 1))
		return;
	if ((x+w > 79) || (y+h > 24))
		return;

	for (int i = y; i < (y+h); i++) 
	{
		gotoxy(x, i);
		for (int j = x; j < (x+w); j++) 
			cout << ' ';
	}
}

// a solid box with a drop shadow
void shadowbox(char x, char y, char w, char h)
{
	HANDLE hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
	CONSOLE_SCREEN_BUFFER_INFO coninfo;
	int color;

	// Get current color
	GetConsoleScreenBufferInfo(hConsole, &coninfo);
	color = coninfo.wAttributes;

	// Set shadow color
	SetConsoleTextAttribute(hConsole, color & 0x000F);

	// Draw the shadow
	solidbox(x+1,y+1,w,h);

	// Reset colors
	SetConsoleTextAttribute(hConsole, color);

	// Draw the box
	solidbox(x,y,w,h);
}

// A double-line box
void doublebox(char x, char y, char w, char h)
{
	char tl = (char)201;
	char tr = (char)187;
	char bl = (char)200;
	char br = (char)188;
	char ve = (char)186;
	char ho = (char)205;
	int i;

	if (w < 2 || h < 2)
		return;

	// draw top
	gotoxy(x,y);
	cout << tl;
	for (i = 0; i < (w-2); i++)
		cout << ho;
	cout << tr;

	// draw sides
	for (i = y+1; i < (y+h-1); i++)
	{
		gotoxy(x, i);
		cout << ve;
		gotoxy(x+w-1, i);
		cout << ve;
	}

	// draw bottom
	gotoxy(x, y+h-1);
	cout << bl;
	for (i = 0; i < (w-2); i++)
		cout << ho;
	cout << br;

	cout << flush;
}
